@@ -187,7 +187,10 @@ class GeometryNode (Node):
                 translation=gt.get_translation(self._cartesianTransform)
                 self._orientedBoundingBox= boundingbox.translate(translation)
         elif self._resource is not None:
-            self._orientedBoundingBox=self._resource.get_oriented_bounding_box()
+            try:
+                self._orientedBoundingBox=self._resource.get_oriented_bounding_box()
+            except:
+                return None
         else:
             return None
         return self._orientedBoundingBox
