@@ -17,6 +17,7 @@ import os
 import open3d as o3d
 import math
 import uuid
+from scipy.spatial.transform import Rotation as R
 
 
 #IMPORT MODULES
@@ -308,6 +309,38 @@ class ImageNode(Node):
     #             return None
     #     return self.orientedBoundingBox
 
+    def get_mesh_geometry(self, depth:float=10, focalLength35mm:float=24)->o3d.geometry.TriangleMesh:
+        """Generate a concical mesh representation using the Image's cartesianTransform and focalLength35mm.
+    
+        .. image:: ../docs/pics/virtual_image2.PNG
+
+        Args:
+            depth (float, optional): Viewing depth of the image. Defaults to 10m.
+            focalLength35mm (float,optional): standardised focal length on 35mm film (w=36mm, h = 24mm)
+
+        Returns:
+            o3d.geometry.TriangleMesh: 
+        """
+        
+        if self.cartesianTransform is not None:
+            radius=35/(focalLength35mm*2)*depth        
+            mesh= o3d.geometry.TriangleMesh.create_cone(radius=radius, height=depth, resolution=20, split=1)
+            rotation=gt.get_rotation_matrix(self.cartesianTransform)
+            r=R.from_matrix(rotation)
+            rz=R.from_euler('xyz' ,[180, 0, 0], degrees=True)
+            t=gt.get_translation(self.cartesianTransform)
+            mesh=mesh.translate(t)
+            r=rz*r
+            # t2=r.as_matrix() * np.array([[1],[0],[0]]) *depth
+            A = np.dot( r.as_matrix(),np.array([0,0,-1]) )*depth
+            mesh=mesh.translate(A)
+            rot=r.as_matrix()
+            mesh=mesh.rotate(rot)
+
+            return mesh
+        else:
+            return None
+
     def get_virtual_image(self, geometries: o3d.geometry, downsampling:int=2)-> o3d.cpu.pybind.geometry.Image:
         pinholeCamera=self.get_pinhole_camera_parameters(downsampling)
         if pinholeCamera is not None:
@@ -381,8 +414,9 @@ class ImageNode(Node):
             getattr(self,'geospatialTransform',None) is not None):
             return True
 
-        pix = PIL.Image.open(self.path) 
-        exifData=ut.get_exif_data(pix)
+        # pix = PIL.Image.open(self.path) 
+        with PIL.Image.open(self.path) as pix:
+            exifData=ut.get_exif_data(pix)
 
         if exifData is not None:
             self.timestamp=ut.get_if_exist(exifData, "DateTime")
@@ -404,6 +438,7 @@ class ImageNode(Node):
                                                 newLongitude,
                                                 ut.get_if_exist(gps_info, "GPSAltitude")]
                     self.coordinateSystem='geospatial-wgs84'
+            
             return True
         else:
             return False
