@@ -178,6 +178,27 @@ def crop_geometry_by_convex_hull(source:trimesh.Trimesh, cutters: List[trimesh.T
     else:
         return None
 
+def get_mesh_representation(geometry)->o3d.geometry.TriangleMesh:
+    """Returns the mesh representation of a o3d.geometry.TriangleMesh or o3d.geometry.PointCloud.\n
+    Returns the convex hull if point cloud.
+
+    Args:
+        geometry (o3d.geometry.TriangleMesh or o3d.geometry.PointCloud)
+
+    Raises:
+        ValueError: 'No valid input: o3d.geometry.TriangleMesh or o3d.geometry.PointCloud'
+
+    Returns:
+        o3d.geometry.TriangleMesh 
+    """
+    if 'PointCloud' in str(type(geometry)):
+        hull, _ =geometry.compute_convex_hull()
+        return hull
+    elif 'TriangleMesh' in str(type(geometry)):
+        return geometry
+    else:
+        raise ValueError('No valid input: o3d.geometry.TriangleMesh or o3d.geometry.PointCloud')
+
 def get_mesh_collisions_trimesh(sourceMesh: o3d.geometry.TriangleMesh, geometries: List[o3d.geometry.TriangleMesh]) -> List[int]:
     """Return indices of geometries that collide with the source.
 
@@ -518,7 +539,7 @@ def generate_virtual_images(geometries: List[o3d.geometry.Geometry],cartesianTra
     """Generate a set of Open3D Images from cartesianTransforms and geometries. 
     The same intrinsic camera parameters are used for all cartesianTransforms that are passed to the function
 
-    .. image:: ../pics/rendering3.PNG
+    .. image:: ../docs/pics/rendering3.PNG
 
     Args:
         geometries (List[o3d.geometry]):o3d.geometry.PointCloud or o3d.geometry.TriangleMesh
@@ -639,6 +660,53 @@ def e57_fix_rotation_order(rotation_matrix:np.array) -> np.array:
                     [r[1,0],r[1,1],-r[1,2]],
                     [r[2,0],r[2,1],-r[2,2]]])
 
+def evaluate_feature(pcd0, pcd1, feat0, feat1, trans_gth, search_voxel_size):
+    """ TO BE EVALUATED"""
+    match_inds = get_matching_indices(pcd0, pcd1, trans_gth, search_voxel_size)
+    pcd_tree = o3d.geometry.KDTreeFlann(feat1)
+    dist = []
+    for ind in match_inds:
+        k, idx, _ = pcd_tree.search_knn_vector_xd(feat0.data[:, ind[0]], 1)
+        dist.append(
+            np.clip(
+                np.power(pcd1.points[ind[1]] - pcd1.points[idx[0]], 2),
+                a_min=0.0,
+                a_max=1.0))
+    return np.mean(dist)
+
+def get_matching_indices(source, target, trans, search_voxel_size, K=None):
+    """ TO BE EVALUATED"""
+
+    source_copy = copy.deepcopy(source)
+    target_copy = copy.deepcopy(target)
+    source_copy.transform(trans)
+    pcd_tree = o3d.geometry.KDTreeFlann(target_copy)
+
+    match_inds = []
+    for i, point in enumerate(source_copy.points):
+        [_, idx, _] = pcd_tree.search_radius_vector_3d(point, search_voxel_size)
+        if K is not None:
+            idx = idx[:K]
+        for j in idx:
+            match_inds.append((i, j))
+    return match_inds
+
+# def get_correspondences(src_pcd, tgt_pcd, trans, search_voxel_size, K=None):
+#     src_pcd.transform(trans)
+#     pcd_tree = o3d.geometry.KDTreeFlann(tgt_pcd)
+
+#     correspondences = []
+#     for i, point in enumerate(src_pcd.points):
+#         [count, idx, _] = pcd_tree.search_radius_vector_3d(point, search_voxel_size)
+#         if K is not None:
+#             idx = idx[:K]
+#         for j in idx:
+#             correspondences.append([i, j])
+    
+#     correspondences = np.array(correspondences)
+#     correspondences = torch.from_numpy(correspondences)
+#     return correspondences
+
 def crop_geometry_by_box(geometry:o3d.geometry, box:o3d.geometry.OrientedBoundingBox, subdivide:int = 0) ->o3d.geometry:
     """"Crop portion of a mesh/pointcloud that lies within an OrientedBoundingBox 
 
@@ -874,6 +942,8 @@ def get_mesh_inliers(sources:o3d.geometry, reference:o3d.geometry) -> List[int]:
 
     sources=ut.item_to_list(sources)
     inliers=[None]*len(sources)
+
+    test=str(type(reference))
     #validate reference
     if ('TriangleMesh' in str(type(reference)) and 
         len(reference.triangles)>=2 and
@@ -911,7 +981,10 @@ def get_mesh_inliers(sources:o3d.geometry, reference:o3d.geometry) -> List[int]:
             inliers[i]=True
             continue
         inliers[i]=False
-    return inliers
+    
+    #select indices    
+    ind=np.where(np.asarray(inliers) ==True)[0]     
+    return ind
 
 def get_box_intersections(sourceBox:o3d.geometry.OrientedBoundingBox, testBoxes: List[o3d.geometry.OrientedBoundingBox])->List[int]:
     """" Return indices of testBoxes of which the geometry intersects with the sourceBox.
